package info.visone.nlpext.crawl;

import java.net.CookieHandler;
import java.net.CookieManager;
import java.net.CookiePolicy;
import java.net.HttpCookie;
import java.net.URI;
import java.net.URL;
import java.util.List;

import info.visone.base.Mediator;
import info.visone.base.Network;
import info.visone.collections.NetworkCollection;
import info.visone.gui.tasks.AbstractAlgorithmTask;
import info.visone.nlpext.crawl.accept.LinkAccepter;
import info.visone.nlpext.crawl.accept.MaxDepth;
import info.visone.nlpext.crawl.accept.NoFollowAccepter;
import info.visone.nlpext.crawl.accept.OnlySameHost;
import info.visone.nlpext.crawl.out.CrawlListener;
import info.visone.nlpext.crawl.sys.Helper4Crawler;
import info.visone.nlpext.crawl.sys.UrlPool;
import info.visone.nlpext.crawl.texter.TexterFactory;
import info.visone.nlpext.crawl.xml.CrawlerRunner;
import info.visone.nlpext.crawl.xml.ProgressProducer;

/**
 * This class sets up a crawler with a given configuration and starts it. The
 * current progress can also be obtained.
 *
 * @see Settings
 * @see ProgressProducer
 * @see CrawlListener
 *
 * @author Joschi
 *
 */
public class CrawlerTask extends AbstractAlgorithmTask {

    /**
     * The thread crawling the net.
     */
    private final CrawlerRunner runner;

    /**
     * Sets up the crawler and starts it. The progress can be obtained via
     * {@link #getProgressProducer()}.
     *
     * @param mediator
     *            The visone mediator
     * @param set
     *            The settings of the crawler.
     * @param start
     *            The first URL to crawl.
     * @param listener
     *            The crawl listener to receive the results.
     * @param followUp
     *            An optional list of URLs to crawl. Usually it is empty and the
     *            following URLs are taken from the <code>start</code>-Page.
     */
    public CrawlerTask(
            final Mediator mediator,
            final Settings set,
            final String start,
            final CrawlListener listener,
            final Object... followUp) {
        super(mediator, "npl.crawler");
        final UrlPool pool = new UrlPool(set.meanDelay, new TexterFactory(set), set.killLimit);
        final URL ustart = Helper4Crawler.getURL(start);
        pool.append(ustart, null);
        for (final Object url : followUp) {
            pool.append(Helper4Crawler.getURL(url.toString()), null);
        }
        pool.addAccepter(new MaxDepth(set.maxDepth));
        if (set.onlySameHost) {
            pool.addAccepter(new OnlySameHost());
        }
        if (set.readNoFollow) {
            pool.addAccepter(new NoFollowAccepter());
        }
        if (set.acceptCookies) {
            final CookiePolicy cp = new CookiePolicy() {

                @Override
                public boolean shouldAccept(final URI uri, final HttpCookie cookie) {
                    return uri.getHost().equals(ustart.getHost())
                            && HttpCookie.domainMatches(cookie.getDomain(), uri.getHost());
                }
            };
            CookieHandler.setDefault(new CookieManager(null, cp));
        }
        for (final LinkAccepter la : set.customAccepter) {
            pool.addAccepter(la);
        }
        if (set.authorizationName != null) {
            if (set.authorizationPassword == null) {
                set.authorizationPassword = "";
            }
            Helper4Crawler.addCustomHeader(
                    "Authorization", "user " + set.authorizationName + ":" + set.authorizationPassword);
        }
        runner = new CrawlerRunner(pool, set, listener);
    }

    /**
     * @return The progress producer of the running crawl.
     */
    public ProgressProducer getProgressProducer() {
        return runner;
    }

    @Override
    public void doWorkInBackground(TaskState arg0) {
        runner.run();
    }

    @Override
    public List<NetworkCollection> getDependentNetworkCollections() {
        return null;
    }

    @Override
    public List<Network> getDependentNetworks() {
        return null;
    }

    @Override
    public boolean prepare(TaskState arg0) {
        return true;
    }

    @Override
    protected boolean doDone(CompletionStatus arg0, TaskState arg1) {
        return true;
    }
}
