package info.visone.nlpext.crawl.gui;

import java.io.IOException;

import javax.swing.SwingUtilities;

import atomicwheel.Graph;
import atomicwheel.why.TransportGraphConverter;
import transportgraph.TransportGraph;

import info.visone.base.Mediator;
import info.visone.base.Network;
import info.visone.ext.nlp.TextNetworkUtil;
import info.visone.gui.tabs.option.BooleanOptionItem;
import info.visone.gui.tabs.option.IntegerOptionItem;
import info.visone.gui.window.VisoneWindow;
import info.visone.nlpext.crawl.CrawlAlgorithm;
import info.visone.nlpext.crawl.Settings;
import info.visone.nlpext.crawl.out.Content;
import info.visone.nlpext.crawl.out.CrawlListener;

/**
 * Crawls a site (or the entire internet), stopping at a given depth or after
 * reaching the page limit, and produces a graph representing the link
 * structure.
 *
 * @author Joschi
 * @author matthias
 */
public class CrawlLinksCard extends AbstractCrawlCard {
    private static final String CARD_NAME = "crawl link structure";
    private IntegerOptionItem maxDepth;
    private IntegerOptionItem limit;
    private BooleanOptionItem onlySameHost;
    private BooleanOptionItem textAsWell;
    private BooleanOptionItem readNoFollow;
    private IntegerOptionItem meanDelay;
    private IntegerOptionItem coolDown;
    private BooleanOptionItem ignoreErrors;

    /**
     * Create the crawler card.
     *
     * @param mediator
     *            the {@link Mediator}
     */
    public CrawlLinksCard(final Mediator mediator) {
        super(CARD_NAME, mediator);
    }

    @Override
    protected boolean wantsIntro() {
        return true;
    }

    @Override
    protected void fillAdditionalImportantOptions() {
        maxDepth = new IntegerOptionItem(2, 0, 20);
        limit = new IntegerOptionItem(1000, 1, 10000, 1000);
        onlySameHost = new BooleanOptionItem();
        textAsWell = new BooleanOptionItem();

        addOptionItem(maxDepth, "max depth");
        addOptionItem(limit, "page limit");
        onlySameHost.setValue(true);
        addOptionItem(onlySameHost, "same host only");
        addOptionItem(textAsWell, "store text");
    }

    @Override
    protected void fillAdditionalObscureOptions() {
        ignoreErrors = new BooleanOptionItem();
        ignoreErrors.setValue(true);
        addOptionItem(ignoreErrors, "ignore errors");
        meanDelay = new IntegerOptionItem(1000, 0, 10000, 1000);
        coolDown = new IntegerOptionItem(5000, 0, 20000, 1000);
        addOptionItem(meanDelay, "mean delay");
        addOptionItem(coolDown, "cool down after error");
        readNoFollow = new BooleanOptionItem();
        readNoFollow.setValue(true);
        addOptionItem(readNoFollow, "respect nofollow");
    }

    @Override
    protected Settings createSettings(final String url) {
        final Settings s = super.createSettings(url);
        s.meanDelay = meanDelay.getValue();
        s.coolDown = coolDown.getValue();
        s.doText = textAsWell.getValue();
        s.onlySameHost = onlySameHost.getValue();
        s.readNoFollow = readNoFollow.getValue();
        s.maxDepth = maxDepth.getValue();
        s.killLimit = limit.getValue();
        return s;
    }

    /** {@inheritDoc} */
    @Override
    public CrawlListener getCrawlListener(final Settings s) {
        final Graph graph = new Graph();
        final CrawlAlgorithm algo = new CrawlAlgorithm(s.doText, graph);

        return new CrawlListener() {
            @Override
            public void pageCrawled(final Content c) {
                algo.processContents(c);
            }

            @Override
            public void close() throws IOException {
                final TransportGraph tg = TransportGraphConverter.createTransportGraph(graph);
                final Network net = TextNetworkUtil.createUnregisteredNetwork(tg, mediator);
                SwingUtilities.invokeLater(() -> {
                    mediator.setNetwork(net);
                    mediator.setStatus("crawling found " + net.getGraph2D().N() + " pages");
                    VisoneWindow window = mediator.getWindow();
                    if (window != null) {
                        window.doModeSelect();
                    }
                });
            }
        };
    }

    @Override
    protected boolean getHaltOnError() {
        return !ignoreErrors.getValue();
    }
}
